use std::sync::atomic::{AtomicU64, AtomicBool, Ordering};
use std::sync::Arc;
use std::time::{Duration, Instant};
use tokio::sync::{Mutex, OnceCell};
use tokio::time::sleep;
use log::{info, warn, error};

/// Connection health status
#[derive(Debug, Clone)]
pub struct ConnectionHealth {
    pub is_healthy: bool,
    pub last_success: Option<Instant>,
    pub last_failure: Option<Instant>,
    pub consecutive_failures: u32,
    pub total_attempts: u64,
    pub total_successes: u64,
    pub total_failures: u64,
    pub uptime_percentage: f64,
}

/// Service type
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
pub enum ServiceType {
    WebSocket,
    WanPolling,
}

impl std::fmt::Display for ServiceType {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            ServiceType::WebSocket => write!(f, "WebSocket"),
            ServiceType::WanPolling => write!(f, "WAN Polling"),
        }
    }
}

/// Health monitor
pub struct HealthMonitor {
    websocket_stats: Arc<ServiceStats>,
    wan_polling_stats: Arc<ServiceStats>,
}

struct ServiceStats {
    is_healthy: AtomicBool,
    last_success: Mutex<Option<Instant>>,
    last_failure: Mutex<Option<Instant>>,
    consecutive_failures: AtomicU64,
    total_attempts: AtomicU64,
    total_successes: AtomicU64,
    total_failures: AtomicU64,
    start_time: Instant,
}

impl ServiceStats {
    fn new() -> Self {
        Self {
            is_healthy: AtomicBool::new(false),
            last_success: Mutex::new(None),
            last_failure: Mutex::new(None),
            consecutive_failures: AtomicU64::new(0),
            total_attempts: AtomicU64::new(0),
            total_successes: AtomicU64::new(0),
            total_failures: AtomicU64::new(0),
            start_time: Instant::now(),
        }
    }

    async fn record_attempt(&self) {
        self.total_attempts.fetch_add(1, Ordering::Relaxed);
    }

    async fn record_success(&self) {
        self.is_healthy.store(true, Ordering::Relaxed);
        self.consecutive_failures.store(0, Ordering::Relaxed);
        self.total_successes.fetch_add(1, Ordering::Relaxed);
        *self.last_success.lock().await = Some(Instant::now());
    }

    async fn record_failure(&self) {
        self.is_healthy.store(false, Ordering::Relaxed);
        self.consecutive_failures.fetch_add(1, Ordering::Relaxed);
        self.total_failures.fetch_add(1, Ordering::Relaxed);
        *self.last_failure.lock().await = Some(Instant::now());
    }

    async fn get_health(&self) -> ConnectionHealth {
        let total_attempts = self.total_attempts.load(Ordering::Relaxed);
        let total_successes = self.total_successes.load(Ordering::Relaxed);
        let total_failures = self.total_failures.load(Ordering::Relaxed);
        
        let uptime_percentage = if total_attempts > 0 {
            (total_successes as f64 / total_attempts as f64) * 100.0
        } else {
            0.0
        };

        ConnectionHealth {
            is_healthy: self.is_healthy.load(Ordering::Relaxed),
            last_success: *self.last_success.lock().await,
            last_failure: *self.last_failure.lock().await,
            consecutive_failures: self.consecutive_failures.load(Ordering::Relaxed) as u32,
            total_attempts,
            total_successes,
            total_failures,
            uptime_percentage,
        }
    }
}

impl HealthMonitor {
    pub async fn global() -> &'static Self {
        static HEALTH_MONITOR: OnceCell<HealthMonitor> = OnceCell::const_new();

        HEALTH_MONITOR
            .get_or_init(|| async {
                let monitor = HealthMonitor::new();
                
                // Start health status reporting task
                let monitor_clone = monitor.clone();
                tokio::spawn(async move {
                    monitor_clone.start_health_reporting().await;
                });
                
                monitor
            })
            .await
    }

    fn new() -> Self {
        Self {
            websocket_stats: Arc::new(ServiceStats::new()),
            wan_polling_stats: Arc::new(ServiceStats::new()),
        }
    }

    fn clone(&self) -> Self {
        Self {
            websocket_stats: self.websocket_stats.clone(),
            wan_polling_stats: self.wan_polling_stats.clone(),
        }
    }

    /// Record service attempt
    pub async fn record_attempt(&self, service: ServiceType) {
        match service {
            ServiceType::WebSocket => self.websocket_stats.record_attempt().await,
            ServiceType::WanPolling => self.wan_polling_stats.record_attempt().await,
        }
    }

    /// Record service success
    pub async fn record_success(&self, service: ServiceType) {
        match service {
            ServiceType::WebSocket => self.websocket_stats.record_success().await,
            ServiceType::WanPolling => self.wan_polling_stats.record_success().await,
        }
    }

    /// Record service failure
    pub async fn record_failure(&self, service: ServiceType) {
        match service {
            ServiceType::WebSocket => self.websocket_stats.record_failure().await,
            ServiceType::WanPolling => self.wan_polling_stats.record_failure().await,
        }
    }

    /// Get service health status
    pub async fn get_health(&self, service: ServiceType) -> ConnectionHealth {
        match service {
            ServiceType::WebSocket => self.websocket_stats.get_health().await,
            ServiceType::WanPolling => self.wan_polling_stats.get_health().await,
        }
    }

    /// Get all services health status
    pub async fn get_all_health(&self) -> (ConnectionHealth, ConnectionHealth) {
        let websocket_health = self.websocket_stats.get_health().await;
        let wan_polling_health = self.wan_polling_stats.get_health().await;
        (websocket_health, wan_polling_health)
    }

    /// Start health status reporting task
    async fn start_health_reporting(&self) {
        let mut interval = tokio::time::interval(Duration::from_secs(60)); // Report every minute
        
        loop {
            interval.tick().await;
            
            let (websocket_health, wan_health) = self.get_all_health().await;
            
            info!("=== Health Status Report ===");
            self.log_service_health("WebSocket", &websocket_health);
            self.log_service_health("WAN Polling", &wan_health);
            
            // Warn if any service is unhealthy
            if !websocket_health.is_healthy {
                warn!("WebSocket service is unhealthy! Consecutive failures: {}", websocket_health.consecutive_failures);
            }
            if !wan_health.is_healthy {
                warn!("WAN Polling service is unhealthy! Consecutive failures: {}", wan_health.consecutive_failures);
            }
            
            // Alert if consecutive failures are too many
            if websocket_health.consecutive_failures > 10 {
                error!("WebSocket service has {} consecutive failures!", websocket_health.consecutive_failures);
            }
            if wan_health.consecutive_failures > 10 {
                error!("WAN Polling service has {} consecutive failures!", wan_health.consecutive_failures);
            }
        }
    }

    fn log_service_health(&self, service_name: &str, health: &ConnectionHealth) {
        info!(
            "{}: {} | Uptime: {:.1}% | Attempts: {} | Successes: {} | Failures: {} | Consecutive Failures: {}",
            service_name,
            if health.is_healthy { "HEALTHY" } else { "UNHEALTHY" },
            health.uptime_percentage,
            health.total_attempts,
            health.total_successes,
            health.total_failures,
            health.consecutive_failures
        );
        
        if let Some(last_success) = health.last_success {
            info!("{}: Last success: {:.1}s ago", service_name, last_success.elapsed().as_secs_f64());
        }
        
        if let Some(last_failure) = health.last_failure {
            info!("{}: Last failure: {:.1}s ago", service_name, last_failure.elapsed().as_secs_f64());
        }
    }
}
