use std::time::Duration;

use anyhow::{anyhow, bail};
use clap::Parser;
use futures_util::{SinkExt, StreamExt};
use log::{debug, error, info, warn};
use tokio::time::timeout;
use tokio_tungstenite::{
    connect_async,
    tungstenite::{
        client::IntoClientRequest,
        protocol::{frame::coding::CloseCode, CloseFrame},
        Message,
    },
};

mod clash_conn_msg;
mod health_monitor;
pub mod retry;
mod statistics;
mod udp_server;
mod wan;

use health_monitor::{HealthMonitor, ServiceType};
use retry::retry_forever;
use wan::poll_wan_traffic;

#[derive(Parser, Debug)]
#[clap(
    version = "0.1.0",
    author = "Ivan Li",
    about = "Watch Clash network traffic"
)]
struct Args {
    #[clap(
        short,
        env,
        long,
        default_value = "ws://192.168.1.1:9090/connections?token=123456"
    )]
    clash_url: String,
    #[clap(short = 'p', long, env, default_value = "17890")]
    listen_port: u16,
    #[clap(short, long, env, default_value = "http://192.168.1.1/cgi-bin/luci")]
    luci_url: String,
    #[clap(short = 'u', long, env, default_value = "root")]
    luci_username: String,
    #[clap(short = 'P', long, env, default_value = "123456")]
    luci_password: String,
}

#[tokio::main]
async fn main() {
    dotenvy::dotenv().ok();
    env_logger::init();

    println!("Hello, world!");

    let args = Args::parse();

    tokio::spawn(async move {
        // Use infinite retry mechanism to ensure WAN traffic monitoring is always available
        retry_forever(|| async {
            let health_monitor = HealthMonitor::global().await;
            health_monitor.record_attempt(ServiceType::WanPolling).await;

            match poll_wan_traffic(
                args.luci_url.as_str(),
                args.luci_username.as_str(),
                args.luci_password.as_str(),
            )
            .await
            {
                Ok(_) => {
                    info!("WAN traffic polling ended normally, restarting...");
                    health_monitor.record_success(ServiceType::WanPolling).await;
                    Ok(())
                }
                Err(err) => {
                    error!("WAN traffic polling failed: {}", err);
                    health_monitor.record_failure(ServiceType::WanPolling).await;
                    Err(err)
                }
            }
        }).await;
    });

    let connect_addr = args.clash_url;

    // Use infinite retry mechanism to ensure WebSocket connection is always available
    retry_forever(|| async {
        let health_monitor = HealthMonitor::global().await;
        health_monitor.record_attempt(ServiceType::WebSocket).await;

        match pipe(connect_addr.clone()).await {
            Ok(_) => {
                info!("WebSocket connection ended normally, reconnecting...");
                health_monitor.record_success(ServiceType::WebSocket).await;
                Ok(())
            }
            Err(err) => {
                error!("WebSocket connection failed: {}", err);
                health_monitor.record_failure(ServiceType::WebSocket).await;
                Err(err)
            }
        }
    }).await;
}

async fn pipe(connect_addr: String) -> anyhow::Result<()> {
    info!("Attempting to connect to WebSocket: {}", connect_addr);

    let request = connect_addr.into_client_request()?;

    // Add connection timeout
    let (mut ws_stream, _) = timeout(Duration::from_secs(30), connect_async(request))
        .await
        .map_err(|_| anyhow!("WebSocket connection timeout after 30 seconds"))?
        .map_err(|err| anyhow!("WebSocket connection failed: {}", err))?;

    info!("WebSocket handshake completed successfully");

    let heartbeat_timeout = Duration::from_secs(60); // Consider connection abnormal if no message for 60 seconds

    loop {
        let message_result = timeout(heartbeat_timeout, ws_stream.next()).await;

        let message = match message_result {
            Ok(Some(Ok(msg))) => msg,
            Ok(Some(Err(err))) => {
                error!("WebSocket message error: {}", err);
                return Err(anyhow!("WebSocket message error: {}", err));
            }
            Ok(None) => {
                warn!("WebSocket stream ended unexpectedly");
                return Err(anyhow!("WebSocket stream ended"));
            }
            Err(_) => {
                error!("WebSocket heartbeat timeout - no message received in {} seconds", heartbeat_timeout.as_secs());
                return Err(anyhow!("WebSocket heartbeat timeout"));
            }
        };

        match message {
            Message::Text(text) => {
                let data = text.as_bytes();

                let wrapper =
                    serde_json::from_slice::<clash_conn_msg::ClashConnectionsWrapper>(&data);

                if let Err(err) = wrapper {
                    error!("parse message failed. {}", err);
                    ws_stream
                        .close(Some(CloseFrame {
                            code: CloseCode::Unsupported,
                            reason: "parse message failed".into(),
                        }))
                        .await
                        .map_err(|err| anyhow!(err))?;
                    bail!(err);
                }
                let wrapper = wrapper.unwrap();

                let statistics_manager = statistics::StatisticsManager::global().await;

                statistics_manager.update(&wrapper).await;

                let state = statistics_manager.get_state().await;

                debug!("len: {},speed_upload: {}, speed_download: {}, update_direct_speed: {}, download_direct_speed: {}, update_proxy_speed: {}, download_proxy_speed: {}",
                state.connections,
                state.speed_upload,
                state.speed_download,
                state.direct_upload_speed,
                state.direct_download_speed,
                state.proxy_upload_speed,
                state.proxy_download_speed,
            );

                let udp_server = udp_server::UdpServer::global().await;
                let mut buf = [0; 32];
                buf[0..8].copy_from_slice(&state.direct_upload_speed.to_le_bytes());
                buf[8..16].copy_from_slice(&state.direct_download_speed.to_le_bytes());
                buf[16..24].copy_from_slice(&state.proxy_upload_speed.to_le_bytes());
                buf[24..32].copy_from_slice(&state.proxy_download_speed.to_le_bytes());
                udp_server.publish_clash(&buf).await;
            }
            Message::Close(close_frame) => {
                if let Some(frame) = close_frame {
                    info!("Server requested close: {} - {}", frame.code, frame.reason);
                } else {
                    info!("Server requested close without reason");
                }
                return Err(anyhow!("WebSocket connection closed by server"));
            }
            Message::Ping(payload) => {
                debug!("Received ping, sending pong");
                // Automatically reply pong to keep connection alive
                if let Err(err) = ws_stream.send(Message::Pong(payload)).await {
                    error!("Failed to send pong: {}", err);
                    return Err(anyhow!("Failed to send pong: {}", err));
                }
            }
            Message::Pong(_) => {
                debug!("Received pong");
            }
            _ => {}
        }
    }

    Ok(())
}
