use std::time::Duration;
use log::{debug, warn, error};
use tokio::time::sleep;

/// Retry strategy configuration
#[derive(Clone, Debug)]
pub struct RetryConfig {
    /// Maximum number of retry attempts
    pub max_attempts: usize,
    /// Initial retry delay
    pub initial_delay: Duration,
    /// Maximum retry delay
    pub max_delay: Duration,
    /// Exponential backoff multiplier
    pub backoff_multiplier: f64,
    /// Whether to enable jitter
    pub jitter: bool,
}

impl Default for RetryConfig {
    fn default() -> Self {
        Self {
            max_attempts: 10,
            initial_delay: Duration::from_millis(500),
            max_delay: Duration::from_secs(30),
            backoff_multiplier: 2.0,
            jitter: true,
        }
    }
}

impl RetryConfig {
    /// Create fast retry configuration (for lightweight operations)
    pub fn fast() -> Self {
        Self {
            max_attempts: 5,
            initial_delay: Duration::from_millis(100),
            max_delay: Duration::from_secs(5),
            backoff_multiplier: 1.5,
            jitter: true,
        }
    }

    /// Create slow retry configuration (for heavyweight operations)
    pub fn slow() -> Self {
        Self {
            max_attempts: 15,
            initial_delay: Duration::from_secs(1),
            max_delay: Duration::from_secs(60),
            backoff_multiplier: 2.0,
            jitter: true,
        }
    }

    /// Create infinite retry configuration (for critical services)
    pub fn infinite() -> Self {
        Self {
            max_attempts: usize::MAX,
            initial_delay: Duration::from_secs(1),
            max_delay: Duration::from_secs(30),
            backoff_multiplier: 2.0,
            jitter: true,
        }
    }
}

/// Retrier implementation
pub struct Retrier {
    config: RetryConfig,
    attempt: usize,
    current_delay: Duration,
}

impl Retrier {
    pub fn new(config: RetryConfig) -> Self {
        Self {
            current_delay: config.initial_delay,
            config,
            attempt: 0,
        }
    }

    /// Execute retry operation
    pub async fn retry<F, Fut, T, E>(&mut self, operation: F) -> Result<T, E>
    where
        F: Fn() -> Fut,
        Fut: std::future::Future<Output = Result<T, E>>,
        E: std::fmt::Display,
    {
        loop {
            self.attempt += 1;

            debug!("Attempting operation (attempt {}/{})", self.attempt, self.config.max_attempts);

            match operation().await {
                Ok(result) => {
                    if self.attempt > 1 {
                        debug!("Operation succeeded after {} attempts", self.attempt);
                    }
                    return Ok(result);
                }
                Err(err) => {
                    if self.attempt >= self.config.max_attempts {
                        error!("Operation failed after {} attempts: {}", self.attempt, err);
                        return Err(err);
                    }

                    warn!("Operation failed (attempt {}/{}): {}", self.attempt, self.config.max_attempts, err);

                    let delay = self.calculate_delay();
                    debug!("Retrying in {:?}", delay);
                    sleep(delay).await;
                }
            }
        }
    }

    /// Reset retrier state
    pub fn reset(&mut self) {
        self.attempt = 0;
        self.current_delay = self.config.initial_delay;
    }

    /// Calculate next retry delay
    fn calculate_delay(&mut self) -> Duration {
        let delay = self.current_delay;

        // Calculate next delay time (exponential backoff)
        let next_delay_ms = (self.current_delay.as_millis() as f64 * self.config.backoff_multiplier) as u64;
        self.current_delay = Duration::from_millis(next_delay_ms).min(self.config.max_delay);

        // Add jitter to avoid thundering herd effect
        if self.config.jitter {
            let jitter_range = delay.as_millis() as f64 * 0.1; // 10% jitter
            let jitter = (rand::random::<f64>() - 0.5) * 2.0 * jitter_range;
            let jittered_delay = (delay.as_millis() as f64 + jitter).max(0.0) as u64;
            Duration::from_millis(jittered_delay)
        } else {
            delay
        }
    }

    /// Get current attempt count
    pub fn attempt_count(&self) -> usize {
        self.attempt
    }
}

/// Convenience function: execute operation with retry
pub async fn retry_with_config<F, Fut, T, E>(
    config: RetryConfig,
    operation: F,
) -> Result<T, E>
where
    F: Fn() -> Fut,
    Fut: std::future::Future<Output = Result<T, E>>,
    E: std::fmt::Display,
{
    let mut retrier = Retrier::new(config);
    retrier.retry(operation).await
}

/// Convenience function: execute retry operation with default configuration
pub async fn retry<F, Fut, T, E>(operation: F) -> Result<T, E>
where
    F: Fn() -> Fut,
    Fut: std::future::Future<Output = Result<T, E>>,
    E: std::fmt::Display,
{
    retry_with_config(RetryConfig::default(), operation).await
}

/// Convenience function: infinite retry (for critical services)
pub async fn retry_forever<F, Fut, T, E>(operation: F) -> T
where
    F: Fn() -> Fut,
    Fut: std::future::Future<Output = Result<T, E>>,
    E: std::fmt::Display,
{
    let mut retrier = Retrier::new(RetryConfig::infinite());
    loop {
        match retrier.retry(&operation).await {
            Ok(result) => return result,
            Err(_) => {
                // This should theoretically never happen since we set infinite retry
                // But for safety, we reset the retrier and continue
                retrier.reset();
                continue;
            }
        }
    }
}

#[cfg(test)]
mod tests {
    use super::*;
    use std::sync::atomic::{AtomicUsize, Ordering};
    use std::sync::Arc;

    #[tokio::test]
    async fn test_retry_success_on_first_attempt() {
        let counter = Arc::new(AtomicUsize::new(0));
        let counter_clone = counter.clone();
        
        let result = retry(|| async {
            counter_clone.fetch_add(1, Ordering::SeqCst);
            Ok::<i32, &str>(42)
        }).await;
        
        assert_eq!(result.unwrap(), 42);
        assert_eq!(counter.load(Ordering::SeqCst), 1);
    }

    #[tokio::test]
    async fn test_retry_success_after_failures() {
        let counter = Arc::new(AtomicUsize::new(0));
        let counter_clone = counter.clone();
        
        let config = RetryConfig {
            max_attempts: 3,
            initial_delay: Duration::from_millis(1),
            max_delay: Duration::from_millis(10),
            backoff_multiplier: 2.0,
            jitter: false,
        };
        
        let result = retry_with_config(config, || async {
            let count = counter_clone.fetch_add(1, Ordering::SeqCst) + 1;
            if count < 3 {
                Err("not ready")
            } else {
                Ok(42)
            }
        }).await;
        
        assert_eq!(result.unwrap(), 42);
        assert_eq!(counter.load(Ordering::SeqCst), 3);
    }

    #[tokio::test]
    async fn test_retry_max_attempts_exceeded() {
        let counter = Arc::new(AtomicUsize::new(0));
        let counter_clone = counter.clone();
        
        let config = RetryConfig {
            max_attempts: 2,
            initial_delay: Duration::from_millis(1),
            max_delay: Duration::from_millis(10),
            backoff_multiplier: 2.0,
            jitter: false,
        };
        
        let result = retry_with_config(config, || async {
            counter_clone.fetch_add(1, Ordering::SeqCst);
            Err::<i32, &str>("always fails")
        }).await;
        
        assert!(result.is_err());
        assert_eq!(counter.load(Ordering::SeqCst), 2);
    }
}
