use std::time::Duration;

use anyhow::anyhow;
use anyhow::bail;
use anyhow::Result;
use http_body_util::BodyExt;
use http_body_util::Empty;
use hyper::StatusCode;
use hyper::{
    body::{Buf, Bytes},
    Request,
};
use hyper_util::rt::TokioIo;
use log::{debug, error, info, warn};
use serde::{Deserialize, Serialize};
use tokio::net::TcpStream;
use tokio::time::{sleep, timeout};

use crate::udp_server;

pub type OpenWRTIFaces = Vec<OpenWRTIFace>;

pub async fn poll_wan_traffic(luci_url: &str, username: &str, password: &str) -> Result<()> {
    info!("Starting WAN traffic polling for: {}", luci_url);

    // Parse our URL...
    let url = format!("{}/admin/network/iface_status/wan", luci_url).parse::<hyper::Uri>()?;

    // Get the host and the port
    let host = url.host().expect("uri has no host");
    let port = url.port_u16().unwrap_or(80);

    let address = format!("{}:{}", host, port);
    info!("Connecting to: {}", address);

    let mut cookies_str = String::new();
    let connection_timeout = Duration::from_secs(10);
    let request_timeout = Duration::from_secs(30);

    // Open a TCP connection to the remote host with timeout
    loop {
        let stream = timeout(connection_timeout, TcpStream::connect(address.clone()))
            .await
            .map_err(|_| anyhow!("Connection timeout after {} seconds", connection_timeout.as_secs()))?
            .map_err(|e| anyhow!("Failed to connect to {}: {}", address, e))?;

        // Use an adapter to access something implementing `tokio::io` traits as if they implement
        // `hyper::rt` IO traits.
        let io = TokioIo::new(stream);

        // Create the Hyper client with timeout
        let (mut sender, conn) = timeout(connection_timeout, hyper::client::conn::http1::handshake(io))
            .await
            .map_err(|_| anyhow!("HTTP handshake timeout after {} seconds", connection_timeout.as_secs()))?
            .map_err(|e| anyhow!("HTTP handshake failed: {}", e))?;

        // Spawn a task to poll the connection, driving the HTTP state
        let conn_handle = tokio::task::spawn(async move {
            if let Err(err) = conn.await {
                error!("HTTP connection failed: {:?}", err);
            }
        });

        let mut prev_up = 0;
        let mut prev_down = 0;
        let mut consecutive_errors = 0;
        const MAX_CONSECUTIVE_ERRORS: u32 = 5;

        loop {
            if sender.is_closed() {
                warn!("HTTP sender is closed, breaking connection loop");
                break;
            }

            if !sender.is_ready() {
                sleep(Duration::from_millis(100)).await;
                continue;
            }

            let poll_req = generate_poll_wan_req(luci_url, &cookies_str)?;

            // Send request with timeout
            let res = timeout(request_timeout, sender.send_request(poll_req.clone()))
                .await
                .map_err(|_| anyhow!("Request timeout after {} seconds", request_timeout.as_secs()))?
                .map_err(|e| anyhow!("Request failed: {}", e))?;

            debug!("Response status: {}", res.status());

            match res.status() {
                StatusCode::FORBIDDEN | StatusCode::UNAUTHORIZED => {
                    info!("Authentication required, logging in...");
                    cookies_str = login(luci_url, username, password).await?;
                    consecutive_errors = 0; // Reset error count
                    continue;
                }
                StatusCode::OK => {
                    consecutive_errors = 0; // Reset error count
                }
                status => {
                    consecutive_errors += 1;
                    error!("Unexpected response status: {} (consecutive errors: {})", status, consecutive_errors);
                    if consecutive_errors >= MAX_CONSECUTIVE_ERRORS {
                        return Err(anyhow!("Too many consecutive errors ({}), giving up", consecutive_errors));
                    }
                    sleep(Duration::from_secs(1)).await;
                    continue;
                }
            }

            // asynchronously aggregate the chunks of the body with timeout
            let body = timeout(request_timeout, res.collect())
                .await
                .map_err(|_| anyhow!("Response body timeout after {} seconds", request_timeout.as_secs()))?
                .map_err(|e| anyhow!("Failed to read response body: {}", e))?
                .aggregate();

            // try to parse as json with serde_json
            let interfaces: OpenWRTIFaces = serde_json::from_reader(body.reader())
                .map_err(|e| anyhow!("Failed to parse JSON response: {}", e))?;

            let wan = interfaces.first()
                .ok_or_else(|| anyhow!("No WAN interface found in response"))?;

            // Detect counter reset (restart scenarios)
            if prev_up > wan.tx_bytes || prev_down > wan.rx_bytes {
                info!("Counter reset detected, resetting baseline values");
                prev_up = wan.tx_bytes;
                prev_down = wan.rx_bytes;
            }

            let up_speed = (wan.tx_bytes - prev_up) as u64;
            let down_speed = (wan.rx_bytes - prev_down) as u64;
            prev_up = wan.tx_bytes;
            prev_down = wan.rx_bytes;

            debug!("WAN traffic: ↑ {} bytes/s, ↓ {} bytes/s", up_speed, down_speed);

            let udp_server = udp_server::UdpServer::global().await;
            let mut buf = [0; 16];
            buf[0..8].copy_from_slice(&up_speed.to_le_bytes());
            buf[8..16].copy_from_slice(&down_speed.to_le_bytes());
            udp_server.publish_wan(&buf).await;

            sleep(Duration::from_secs(1)).await;
        }
    }
}

async fn login(luci_url: &str, username: &str, password: &str) -> Result<String> {
    info!("Attempting to login to: {}", luci_url);

    let url = format!("{}/admin/network/iface_status/wan", luci_url).parse::<hyper::Uri>()?;
    let host = url.host().expect("uri has no host");
    let port = url.port_u16().unwrap_or(80);

    let address = format!("{}:{}", host, port);
    let connection_timeout = Duration::from_secs(10);
    let request_timeout = Duration::from_secs(30);

    let stream = timeout(connection_timeout, TcpStream::connect(address.clone()))
        .await
        .map_err(|_| anyhow!("Login connection timeout after {} seconds", connection_timeout.as_secs()))?
        .map_err(|e| anyhow!("Failed to connect for login to {}: {}", address, e))?;

    let io = TokioIo::new(stream);
    let (mut sender, conn) = timeout(connection_timeout, hyper::client::conn::http1::handshake(io))
        .await
        .map_err(|_| anyhow!("Login handshake timeout after {} seconds", connection_timeout.as_secs()))?
        .map_err(|e| anyhow!("Login handshake failed: {}", e))?;

    let _conn_handle = tokio::task::spawn(async move {
        if let Err(err) = conn.await {
            error!("Login connection failed: {:?}", err);
        }
    });

    let login_req = generate_login_req(luci_url, username, password)?;
    let res = timeout(request_timeout, sender.send_request(login_req.clone()))
        .await
        .map_err(|_| anyhow!("Login request timeout after {} seconds", request_timeout.as_secs()))?
        .map_err(|e| anyhow!("Login request failed: {}", e))?;

    match res.status() {
        StatusCode::OK | StatusCode::FOUND | StatusCode::SEE_OTHER => {
            info!("Login successful");
        }
        StatusCode::FORBIDDEN | StatusCode::UNAUTHORIZED => {
            error!("Login failed: Invalid credentials");
            bail!("Login failed with status: {}", res.status());
        }
        status => {
            error!("Login failed with unexpected status: {}", status);
            bail!("Login failed with status: {}", status);
        }
    }

    let cookies = res.headers().get_all(hyper::header::SET_COOKIE);

    let cookies = cookies
        .iter()
        .filter_map(|cookie| cookie.to_str().ok())
        .filter_map(|cookie| cookie.split(';').next())
        .collect::<Vec<_>>();

    if cookies.is_empty() {
        warn!("No cookies received from login response");
    }

    let cookies_str = cookies.join("; ");
    debug!("Received cookies: {}", cookies_str);

    Ok(cookies_str)
}

fn generate_poll_wan_req(luci_url: &str, cookie: &str) -> Result<Request<Empty<Bytes>>> {
    let url = format!("{}/admin/network/iface_status/wan", luci_url).parse::<hyper::Uri>()?;
    let authority = url.authority().unwrap().clone();
    let target = url.path_and_query().unwrap().clone();

    debug!("Polling WAN traffic. Cookie: {:?}", cookie);
    return Request::builder()
        .uri(target)
        .header(hyper::header::HOST, authority.as_str())
        .header(hyper::header::CONNECTION, "Keep-Alive")
        .header(hyper::header::COOKIE, cookie)
        .body(Empty::<Bytes>::new())
        .map_err(|e| anyhow!(e));
}

fn generate_login_req(luci_url: &str, username: &str, password: &str) -> Result<Request<String>> {
    let url = format!("{}/admin/network/iface_status/wan", luci_url).parse::<hyper::Uri>()?;
    let authority = url.authority().unwrap().clone();
    let target = url.path_and_query().unwrap().clone();

    let login_params = format!("luci_username={}&luci_password={}", username, password);
    let body = login_params;

    return Request::builder()
        .uri(target)
        .method("POST")
        .header(hyper::header::HOST, authority.as_str())
        .header(
            hyper::header::CONTENT_TYPE,
            "application/x-www-form-urlencoded",
        )
        .header(hyper::header::CONNECTION, "Keep-Alive")
        .body(body)
        .map_err(|e| anyhow!(e));
}

#[derive(Serialize, Deserialize)]
pub struct OpenWRTIFace {
    rx_bytes: u64,
    tx_bytes: u64,
}

impl Default for OpenWRTIFace {
    fn default() -> Self {
        Self {
            rx_bytes: 0,
            tx_bytes: 0,
        }
    }
}
